/**
 * @file   test.c
 * @author Marc Munro
 * @date   Fri Jul  8 13:20:51 2005
 * 
 * @brief  Test for doxygen reference generation.  Check the diffs
 * between the refs and graphs created for veil_init_range() and
 * veil_init_range2()
 * 
 * 
 */

// Dimitri: Comment out the following line and all is well with the
// output.  I have included my copy of postgres

#include "postgres.h"
//#include "spi.h"
//#include "funcapi.h"
//#include "utils/hsearch.h"

//#include "veil_funcs.h"
//#include "veil_datatypes.h"

/** 
 * Create a dynamically allocated C string as a copy of a text value.
 * 
 * @param in text value from which the copy is made.
 * @return Dynamically allocated (by palloc()) copy of in.
 */
static char *
strfromtext(text *in)
{
    char *out = palloc(VARSIZE(in) - VARHDRSZ + 1);
    memcpy(out, VARDATA(in), VARSIZE(in) - VARHDRSZ);
    out[VARSIZE(in) - VARHDRSZ] = '\0';

    return out;
}

/** 
 * Perform session initialisation once for the session.  This calls the
 * user-defined function veil_init which should create and possibly
 * initialise all shared and session variables.  This function may be
 * safelt called any number of times - it will only perform the
 * initialisation on the first call.
 * 
 */
static void
ensure_init()
{
    bool success;
    int  ok;
    static bool todo = true;

    if (todo) {
        ok = SPI_connect();
        if (ok != SPI_OK_CONNECT) {
            elog(ERROR, "Failed to initialise session (1)");
        }

        todo = false;
        (void) bool_from_query("select veil_init(FALSE)", &success);

        if (!success) {
            elog(ERROR, "Failed to initialise session (2)");
        }
        
        ok = SPI_finish();
        if (ok != SPI_OK_FINISH) {
            elog(ERROR, "Failed to initialise session (3)");
        }
    }
}

/** 
 * Return the Range variable matching the name parameter, possibly
 * creating the variable.  Raise an error if the named variable already
 * exists and is of the wrong type.
 * 
 * @param name The name of the variable.
 * @param create Whether to create the variable if it does not exist.
 * @return Pointer to the variable or null if the variable does not
 * exist and create was false.
 */
Range *
GetRange(char *name,
         bool  create)
{
    VarEntry *var;
    Range    *range;
    
    var = vl_lookup_variable(name);
    range = (Range *) var->obj;

    if (range) {
        if (range->type != OBJ_RANGE) {
            type_mismatch(name, OBJ_RANGE, range->type);
        }
    }
    else {
        if (create) {
            var->obj = (Object *) NewRange(var->shared);
            range = (Range *) var->obj;
        }
        else {
            type_mismatch(name, OBJ_RANGE, OBJ_UNDEFINED);
        }
    }
    return range;
}

PG_FUNCTION_INFO_V1(veil_init_range2);
/** 
 * veil_init_range2(text, int4, int4) returns int4
 * Initialise a Range variable.
 *
 * @param fcinfo <code>text</code> The name of the variable to
 * initialise.
 * <br><code>int4</code> The min value of the range.
 * <br><code>int4</code> The max value of the range.
 * @return <code>int4</code> The size of the range ((max - min) + 1).
 */
Datum
veil_init_range2(PG_FUNCTION_ARGS)
{
    int32     min;
    int32     max;
    Range    *range;
    char     *name;

	ensure_init();
    name = strfromtext(PG_GETARG_TEXT_P(0));
    min = PG_GETARG_INT32(1);
    max = PG_GETARG_INT32(2);

	range = GetRange(name, true);

    range->min = min;
    range->max = max;
    PG_RETURN_INT32(max + 1 - min);
	}
}

PG_FUNCTION_INFO_V1(veil_init_range);
/** 
 * veil_init_range(text, int4, int4) returns int4
 * Initialise a Range variable.
 *
 * @param fcinfo <code>text</code> The name of the variable to
 * initialise.
 * <br><code>int4</code> The min value of the range.
 * <br><code>int4</code> The max value of the range.
 * @return <code>int4</code> The size of the range ((max - min) + 1).
 */
Datum
veil_init_range(PG_FUNCTION_ARGS)
{
    int32     min;
    int32     max;
    Range    *range;
    char     *name;

	/**/ 
	{
	ensure_init();
    name = strfromtext(PG_GETARG_TEXT_P(0));
    min = PG_GETARG_INT32(1);
    max = PG_GETARG_INT32(2);

	range = GetRange(name, true);

    range->min = min;
    range->max = max;
    PG_RETURN_INT32(max + 1 - min);
	}
}
